// ====================== EIGENSCHAFTEN DES INDIKATORS (I) ========================================
// Property-Parameter: Allgemeine Eigenschaften des Indikators
#property copyright "Copyright  2011, Serge Kruse. SKYPE: ckateptb"
#property link      "serge.kruse@gmail.com"
#property indicator_chart_window    // Anzeige im Chartfenster
#property indicator_buffers 3       // Anzahl der Puffers fr die Werte der Indikatorenvariablen
//#property indicator_color1 Yellow   // Farbe der Indikatorlinie



// ====================== VARIABLEN ======================

// Wellen zhlen
int start_koordinaten_einmal = 0;      // Anfangskoordinaten einmalig festzulegen
int welle_pro_bar_einmal;              // pro bar nur eine Welle zeichnen.
string wellerichtung = "a";            // "l" - long; "s" - short; "a" - anfang
extern int wpr_periode = 21;
extern int wpr_ebenen_oben = -20;      // extreme WPR-Zone
extern int wpr_ebenen_unten = -80;     // extreme WPR-Zone
extern bool v_linie = true;            // v_linie zeichnen, wenn die vorangegangene Welle aufgezeichnet wurde

// Koordinaten der Wellen-Linie
datetime wellenzeit1;
double wellenpreis1;
datetime wellenzeit2;
double wellenpreis2;



// ====================== EIGENSCHAFTEN DES INDIKATORS (II) ======================
// buffers des Indikators
double IndPuffer1_a[];        // Koordinate A der vorangegangenen Welle
double IndPuffer2_b[];        // Koordinate B der vorangegangenen Welle
double IndPuffer3_richtung[]; // Richtung der vorangegangenen Welle

// Initiierung des Indikators
int init()
{
   //---- Puffer mit dem Indikator ber Index mappen
   SetIndexBuffer(0,IndPuffer1_a);
   SetIndexBuffer(1,IndPuffer2_b);
   SetIndexBuffer(2,IndPuffer3_richtung);
   // Stil der Linie
   SetIndexStyle(0,DRAW_NONE,EMPTY,1,Yellow);
   SetIndexStyle(1,DRAW_NONE,EMPTY,1,Yellow);
   SetIndexStyle(2,DRAW_NONE,EMPTY,1,Yellow);
   // Kurzname des Indikators
   IndicatorShortName("iWellen");
   // Kurznamen fr die Buffers
   SetIndexLabel(0,"Welle_Point_A");
   SetIndexLabel(1,"Welle_Point_B");
   SetIndexLabel(2,"Welle_Richtung");
   // ab welchem Bar wird es gezeichnet
   SetIndexDrawBegin(0,50);
   SetIndexDrawBegin(1,50);
   SetIndexDrawBegin(2,50);
   // wenn Buffer kein Value liefert, dann 0.0
   SetIndexEmptyValue(0,0.0);
   SetIndexEmptyValue(1,0.0);
   SetIndexEmptyValue(2,0.0);
   
   // initialization done
   return(0);
}



// ====================== FUNKTIONEN ======================
// Funktion: Welle zeichnen.
void zeichneWelle(datetime zeit1, double preis1, datetime zeit2, double preis2, color farbe, string wellen_ebene, datetime bartime, string w_richtung, int barNr)
{
   //Objekt generieren (Trendlinie)
   string s_obj_name = barNr;
   ObjectCreate(s_obj_name, OBJ_TREND, 0, zeit1, preis1, zeit2, preis2);
   //Objekt formatieren
   ObjectSet(s_obj_name, OBJPROP_COLOR, farbe);    // Farbe des Symbols
   ObjectSet(s_obj_name, OBJPROP_RAY, 0);          // Trendlinie ist kein Strahl
   ObjectSet(s_obj_name, OBJPROP_WIDTH, 2);        // Trendlinie Breite
   
   //Objekt generieren (V-Linie)
   if (v_linie == true)
   {
      string s_obj_name_1 = "V-Linie_"+barNr;
      ObjectCreate(s_obj_name_1, OBJ_VLINE, 0, bartime, 0);
      //Objekt formatieren
      ObjectSet(s_obj_name_1, OBJPROP_COLOR, farbe);        // Farbe des Symbols
      ObjectSet(s_obj_name_1, OBJPROP_RAY, 0);              // Trendlinie ist kein Strahl
      ObjectSet(s_obj_name_1, OBJPROP_WIDTH, 0);            // Trendlinie Breite
      ObjectSet(s_obj_name_1, OBJPROP_STYLE, STYLE_DASH);   // Trendlinie Style (nur wenn die Breite nicht grsser als 1 !!!)
   }
   
   // jetzt noch Buffer-Werte anpassen:
   IndPuffer1_a[barNr] = preis1;
   IndPuffer2_b[barNr] = preis2;
   if (w_richtung == "a") {IndPuffer3_richtung[barNr] = 0;}    // da indikatoren nur mit double gehen ...
   if (w_richtung == "l") {IndPuffer3_richtung[barNr] = 1;}
   if (w_richtung == "s") {IndPuffer3_richtung[barNr] = -1;}
   /*Print("A=> "+preis1);
   Print("B=> "+preis2);
   Print("R=> "+w_richtung);*/
}




// ====================== START (bei der Ankunft jedes neuen Ticks/Bars) ======================
int start()
  {
   int i;                                 // Nummer des Bars, der berechnet werden soll
   int counted_bars=IndicatorCounted();   // Anzahl der bereits endgltig berechneter Bars
   if(counted_bars<0) return(-1);         // falls Fehler -> "false"
   if(counted_bars>0) counted_bars--;     // Anzahl der bereits endgltig berechneten Bars - 1.
   i=Bars - counted_bars;                 // der lteste noch nicht berechnete Bar+1  soll berechnet werden.
   
   while(i >= 0)
   {	
      // ---------------------------------------------------------
      // Indikatorenwerte fr den Bar auslesen:
      // ---------------------------------------------------------
      double wpr_akt = iWPR(NULL,0,wpr_periode,i);
      //double wpr_vor = iWPR(NULL,0,wpr_periode,i+1);
      
      
      
      // ---------------------------------------------------------
      // Koordinaten fr die erste Welle einmalig festlegen:
      // ---------------------------------------------------------
      if (start_koordinaten_einmal == 0)
      {
         wellenzeit1 = Time[i];
         wellenpreis1 = Close[i];
         wellenzeit2 = Time[i];
         wellenpreis2 = Close[i];
         start_koordinaten_einmal = 1;    // ab hier werden die Koordinaten-Werte immer automatisch berechnet   
      }
      
      
      
      // ---------------------------------------------------------
      // zweite Zeit- und Preiskoordinaten permanent anpassen
      // ---------------------------------------------------------
      if (
            High[i] > wellenpreis2   // long
            && (wellerichtung == "l" || wellerichtung == "a")
         )
      {
         wellenpreis2 = High[i];
         wellenzeit2 = Time[i];
      }
      if (
            Low[i] < wellenpreis2   // short
            && (wellerichtung == "s" || wellerichtung == "a")
         )
      {
         wellenpreis2 = Low[i];
         wellenzeit2 = Time[i];
      }
      
      
      
      // ---------------------------------------------------------
      // Wellen zhlen/zeichnen
      // ---------------------------------------------------------
      
      // WPR oben. vorangegangene short-Welle abschliessen
      if (
            wpr_akt > wpr_ebenen_oben
            && (wellerichtung == "s" || wellerichtung == "a")
            && welle_pro_bar_einmal != i   // nur einmal pro Bar zeichnen/anpassen.
         )
      {
         // vorangegangene short-Welle zeichnen
         zeichneWelle(wellenzeit1, wellenpreis1, wellenzeit2, wellenpreis2, Yellow, "0", Time[i], wellerichtung, i);
         if(ObjectFind(""+i)!=0) // Index des Fensters mit dem gesuchten Objekt
         {
            Print("Short-Welle konnte nicht generiert werden #",GetLastError());
            return(0);  // Objekt nicht generiert.
         }
         else
         {
            // fr die nchste Welle anpassen
            wellerichtung = "l";          // ab jetzt Welle nach oben
            wellenzeit1 = wellenzeit2;    // erste Zeitkoordinate: bleibt
            wellenpreis1 = wellenpreis2;  // erste Preiskoordinate: bleibt
            wellenzeit2 = Time[i];        // zweite Zeitkoordinate: permanent anpassen
            wellenpreis2 = High[i];       // zweite Preiskoordinate: permanent anpassen
            //Dieser Bar bekommt keine Wellen mehr, falls WPR starke Ausschlge bekommt.
            welle_pro_bar_einmal = i;
         }
      }
      // WPR unten. vorangegangene long-Welle abschliessen
      if (
            wpr_akt < wpr_ebenen_unten
            && (wellerichtung == "l" || wellerichtung == "a")
            && welle_pro_bar_einmal != i   // nur einmal pro Bar zeichnen/anpassen.
         )
      {
         // vorangegangene long-Welle zeichnen
         zeichneWelle(wellenzeit1, wellenpreis1, wellenzeit2, wellenpreis2, Yellow, "0", Time[i], wellerichtung, i);
         if(ObjectFind(""+i)!=0) // Index des Fensters mit dem gesuchten Objekt
         {
            Print("Long-Welle konnte nicht generiert werden #",GetLastError());
            return(0);  // Objekt nicht generiert.
         }
         else
         {
            // fr die nchste Welle anpassen
            wellerichtung = "s";          // ab jetzt Welle nach unten
            wellenzeit1 = wellenzeit2;    // erste Zeitkoordinate: bleibt
            wellenpreis1 = wellenpreis2;  // erste Preiskoordinate: bleibt
            wellenzeit2 = Time[i];        // zweite Zeitkoordinate: permanent anpassen
            wellenpreis2 = Low[i];        // zweite Preiskoordinate: permanent anpassen
            //Dieser Bar bekommt keine Wellen mehr, falls WPR starke Ausschlge bekommt.
            welle_pro_bar_einmal = i;
         }
      }
      
      
      
      i--;
   }  // Ende der while-Schleife.
   //fertig
   return(0);
   
   
   
}  // Ende von START